CREATE TABLE IF NOT EXISTS `{table}`
(
    `player_uuid` CHAR(36) CHARACTER SET ascii COLLATE ascii_bin NOT NULL,
    `id` VARCHAR(50) CHARACTER SET ascii COLLATE ascii_bin NOT NULL,
    `earn_date` TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
    UNIQUE `player_uuid_index` (`player_uuid`, `id`) USING BTREE
);

-- Create stored procedure used to loop through passed achievements
-- After testing, this is the most efficient way to update player properties in MySQL
-- Transactions and batches were 2x slower in my testing
DROP procedure IF EXISTS `mbedwars_player_achievements_update`;

CREATE PROCEDURE `mbedwars_player_achievements_update`(_player_uuid CHAR (36) CHARACTER SET ascii COLLATE ascii_bin,
                                                       _raw_achievements MEDIUMTEXT)
proced: BEGIN
	-- init everything
    DECLARE _end_index INT;
    DECLARE _id TEXT DEFAULT NULL;
    DECLARE _value TEXT DEFAULT NULL;

    -- exit the proc if the list is empty
    if LENGTH(_raw_achievements) = 0 then
        DELETE FROM `{table}`
            WHERE `player_uuid` = _player_uuid;
        LEAVE proced;
    end if;

    -- ok, we want some to be added
    DROP TABLE IF EXISTS _removal_achievements;
    CREATE TEMPORARY TABLE _removal_achievements
        (`id` VARCHAR(50) CHARACTER SET ascii COLLATE ascii_bin NOT NULL)
        ENGINE=MEMORY;
    INSERT INTO _removal_achievements
        SELECT `id`
        FROM `{table}`
        WHERE `player_uuid` = _player_uuid;

    -- ### first go through all achievements
    iterator: LOOP
        -- capture the next value from the list
        SET _end_index = POSITION(',' IN _raw_achievements);

        if _end_index = 0 then
            SET _end_index = LENGTH(_raw_achievements);
        else
            SET _end_index = _end_index - 1;
        end if;
        
        -- parse the entry
        SET _id = SUBSTRING_INDEX(_raw_achievements, '!', 1);
        SET _value = STR_TO_DATE(SUBSTR(_raw_achievements, LENGTH(_id)+2, _end_index-LENGTH(_id)-1), '%Y-%m-%dT%TZ');
        
        -- update table
        REPLACE
            INTO `{table}` (`player_uuid`, `id`, `earn_date`)
            VALUES (_player_uuid, _id, _value);

        -- tell memory table that we don't want to remove this entry
        DELETE
            FROM _removal_achievements
            WHERE `id` = _id;

        -- we are done!
        if _end_index = LENGTH(_raw_achievements) then
            LEAVE iterator;
        end if;

        -- remove the part of the string we just processed
        SET _raw_achievements = SUBSTR(_raw_achievements, _end_index+2);
    END LOOP;

    -- ### remove left properties
    DELETE
    FROM `{table}`
    WHERE `player_uuid` = _player_uuid
      AND `id` IN (SELECT * FROM _removal_achievements);
END;