-- This file contains queries that use avoid using functions from newer SQL versions
-- ----------------------------------------------------


-- Create stored procedure used to gets stats from a player with their uuid
DROP procedure IF EXISTS `mbedwars_player_stats_get_by_uuid`;
CREATE PROCEDURE `mbedwars_player_stats_get_by_uuid`(_player_uuid CHAR(36) CHARACTER SET ascii COLLATE ascii_bin,
                                                     _sort_stat_id VARCHAR(50) CHARACTER SET ascii COLLATE ascii_bin)
BEGIN
	SET @rank = 0;

    SELECT *,
           (SELECT rank
            FROM (SELECT `player_uuid`,
                         @rank := IFNULL(@rank,0)+1 AS `rank`
                  FROM `{table}`
                  WHERE `key` = _sort_stat_id
                  ORDER BY `value` DESC) AS T
            WHERE T.`player_uuid` = _player_uuid) AS `rank`
    FROM `{table}`
    WHERE `player_uuid` = _player_uuid;
END;

-- Create stored procedure used to gets stats from a player with their name
DROP procedure IF EXISTS `mbedwars_player_stats_get_by_name`;
CREATE PROCEDURE `mbedwars_player_stats_get_by_name`(_username VARCHAR (16) CHARACTER SET ascii COLLATE ascii_bin,
                                                     _sort_stat_id VARCHAR (50) CHARACTER SET ascii COLLATE ascii_bin)
BEGIN
	SET @rank = 0;

	-- get uuid by name
	-- will also sort by last login when there are multiple entries with the same name
	SET @uuid = (SELECT `player_uuid` FROM `{table_names}` WHERE `username` = _username);

    SELECT *,
           (SELECT rank
            FROM (SELECT `player_uuid`,
                         @rank := IFNULL(@rank,0)+1 AS `rank`
                  FROM `{table}`
                  WHERE `key` = _sort_stat_id
                  ORDER BY `value` DESC) AS T
            WHERE T.`player_uuid` = @uuid) AS `rank`
    FROM `{table}`
    WHERE `player_uuid` = @uuid;
END;

-- Create stored procedure used to fetch leaderboard ranked players stats+properties from leaderboard
DROP PROCEDURE IF EXISTS `mbedwars_player_fetch_leaderboard`;
CREATE PROCEDURE `mbedwars_player_fetch_leaderboard` (_stat_key VARCHAR (50) CHARACTER SET ascii COLLATE ascii_bin,
                                                      _offset INT,
                                                      _length INT)
BEGIN
    -- get stats
    (SELECT tfinal.`player_uuid`, tfinal.`key`, tfinal.`value`, "stats" AS `source`
     FROM `{table}` tfinal,
          (SELECT *
           FROM `{table}`
           WHERE `key` = _stat_key
           ORDER BY value DESC LIMIT _offset, _length) tcontains
     WHERE tfinal.player_uuid = tcontains.player_uuid)

    UNION

    -- get properties
    (SELECT tfinal.`player_uuid`, tfinal.`id` AS `key`, tfinal.`value`, "properties" AS `source`
     FROM `{table_properties}` tfinal,
          (SELECT *
           FROM `{table}`
           WHERE `key` = _stat_key
           ORDER BY value DESC LIMIT _offset, _length) tcontains
     WHERE tfinal.player_uuid = tcontains.player_uuid);
END;