-- This file contains queries that use functions that don't exist in older SQL versions
-- They greatly improve the performance
-- ----------------------------------------------------

-- Create stored procedure used to gets stats from a player with their uuid
DROP procedure IF EXISTS `mbedwars_player_stats_get_by_uuid`;
CREATE PROCEDURE `mbedwars_player_stats_get_by_uuid`(_player_uuid CHAR (36) CHARACTER SET ascii COLLATE ascii_bin,
                                                     _sort_stat_id VARCHAR (50) CHARACTER SET ascii COLLATE ascii_bin)
BEGIN
    SELECT
        *,
        RANK() OVER (
            PARTITION BY `key` = _sort_stat_id
            ORDER BY `value` DESC
        ) AS `rank`
    FROM `{table}`
    WHERE `player_uuid` = _player_uuid;
END;

-- Create stored procedure used to gets stats from a player with their name
DROP procedure IF EXISTS `mbedwars_player_stats_get_by_name`;
CREATE PROCEDURE `mbedwars_player_stats_get_by_name`(_username VARCHAR (16) CHARACTER SET ascii COLLATE ascii_bin,
                                                     _sort_stat_id VARCHAR (50) CHARACTER SET ascii COLLATE ascii_bin)
BEGIN
	-- get uuid by name
	-- will also sort by last login when there are multiple entries with the same name
	SET @uuid = (SELECT `player_uuid` FROM `{table_names}` WHERE `username` = _username);

    SELECT
        *,
        RANK() OVER (
            PARTITION BY `key` = _sort_stat_id
            ORDER BY `value` DESC
        ) AS `rank`
    FROM `{table}`
    WHERE `player_uuid` = @uuid;
END;

-- Create stored procedure used to fetch leaderboard ranked players stats+properties from leaderboard
DROP PROCEDURE IF EXISTS `mbedwars_player_fetch_leaderboard`;
CREATE PROCEDURE `mbedwars_player_fetch_leaderboard` (_stat_key VARCHAR (50) CHARACTER SET ascii COLLATE ascii_bin,
                                                      _offset INT,
                                                      _length INT)
BEGIN
    WITH tcontains AS (
        SELECT `player_uuid`
        FROM `{table}`
        WHERE `key` = _stat_key
        ORDER BY `value` DESC
        LIMIT _offset, _length
    )

    -- get stats
    SELECT tfinal.`player_uuid`, tfinal.`key`, tfinal.`value`, "stats" AS `source`
        FROM `{table}` tfinal
        JOIN tcontains ON tfinal.player_uuid = tcontains.player_uuid

    UNION

    -- get properties
    SELECT tfinal.`player_uuid`, tfinal.`id` AS `key`, tfinal.`value`, "properties" AS `source`
        FROM `{table_properties}` tfinal
        JOIN tcontains ON tfinal.player_uuid = tcontains.player_uuid;
END;