-- Create table
CREATE TABLE IF NOT EXISTS `{table}`
(
    `player_uuid`
        CHAR (36)
        CHARACTER SET ascii
        COLLATE ascii_bin
        NOT NULL,
    `key`
        VARCHAR (50)
        CHARACTER SET ascii
        COLLATE ascii_bin
        NOT NULL,
    `value`
        DECIMAL (20, 5)
        NOT NULL,
    UNIQUE `player_uuid_index` (`player_uuid`, `key`) USING BTREE,
    INDEX `ranking_index` (`key`, `value`) USING HASH
);

-- Create stored procedure used to loop through passed stats
-- After testing, this is the most efficient way to update player properties in MySQL
-- Transactions and batches were 2x slower in my testing
DROP PROCEDURE IF EXISTS `mbedwars_player_stats_update`;

CREATE PROCEDURE `mbedwars_player_stats_update`(_player_uuid CHAR(36) CHARACTER SET ascii COLLATE ascii_bin,
                                                _raw_stats MEDIUMTEXT)
proced: BEGIN
	-- init everything
    DECLARE _end_index INT;
    DECLARE _key TEXT DEFAULT NULL;
    DECLARE _value DECIMAL(20,10) DEFAULT NULL;

    -- exit the proc if the list is empty
    if LENGTH(_raw_stats) = 0 then
        DELETE FROM `{table}`
            WHERE `player_uuid` = _player_uuid;
        LEAVE proced;
    end if;

    -- ok, we want some to be added
    DROP TABLE IF EXISTS _removal_stats;
    CREATE TEMPORARY TABLE _removal_stats
        (`key` VARCHAR(50) CHARACTER SET ascii COLLATE ascii_bin NOT NULL)
        ENGINE=MEMORY;
    INSERT INTO _removal_stats
        SELECT `key`
        FROM `{table}`
        WHERE `player_uuid` = _player_uuid;

    -- ### first go through all stats
    iterator: LOOP
        -- capture the next value from the list
        SET _end_index = POSITION(',' IN _raw_stats);

        if _end_index = 0 then
            SET _end_index = LENGTH(_raw_stats);
        else
            SET _end_index = _end_index - 1;
        end if;

        -- parse the entry
        SET _key = SUBSTRING_INDEX(_raw_stats, '!', 1);
        SET _value = CAST(SUBSTR(_raw_stats, LENGTH(_key)+2, _end_index-LENGTH(_key)-1) AS DECIMAL(20, 10));

        -- update table
        REPLACE
            INTO `{table}` (`player_uuid`, `key`, `value`)
            VALUES (_player_uuid, _key, _value);

        -- tell memory table that we don't want to remove this entry later
        DELETE
            FROM _removal_stats
            WHERE `key` = _key;

        -- we are done!
        if _end_index = LENGTH(_raw_stats) then
            LEAVE iterator;
        end if;

        -- remove the part of the string we just processed
        SET _raw_stats = SUBSTR(_raw_stats, _end_index+2);
    END LOOP;

    -- ### remove left stats
    DELETE
        FROM `{table}`
        WHERE `player_uuid` = _player_uuid
          AND `key` IN (SELECT * FROM _removal_stats);
END;